#!/bin/zsh

# Author: Kevin Hunter Kesling (original Python)
# Shell Port by: Andrew Kloosterhuis (zsh scripted)

set -e

# Usage message
usage() {
  echo "Usage: $0 --token <auth_token> --host <fw_host> --mapping <auth_username.csv> [--port <port>]"
  exit 1
}

# Default port
PORT=20443

# Parse arguments
while [[ $# -gt 0 ]]; do
  case "$1" in
    --token)
      TOKEN="$2"
      shift 2
      ;;
    --host)
      HOST="$2"
      shift 2
      ;;
    --mapping)
      MAPPING="$2"
      shift 2
      ;;
    --port|-p)
      PORT="$2"
      shift 2
      ;;
    *)
      echo "Unknown argument: $1"
      usage
      ;;
  esac
done

# Validate required arguments
[[ -z "$TOKEN" || -z "$HOST" || -z "$MAPPING" ]] && usage

# Encode token in base64
ENCODED_TOKEN=$(echo -n "$TOKEN" | base64 | tr -d '\n')

# Validate CSV exists
if [[ ! -f "$MAPPING" ]]; then
  echo "ERROR: File not found - $MAPPING"
  exit 1
fi

# Validate header
HEADER=$(head -n1 "$MAPPING" | tr -d '\r')
EXPECTED_HEADER="Device ID,Enrollment Username"
if [[ "$HEADER" != "$EXPECTED_HEADER" ]]; then
  echo "ERROR: Invalid CSV header. Expected: '$EXPECTED_HEADER', Got: '$HEADER'"
  exit 1
fi

# Init result trackers
unexpected_success=()
failures=()
noted_usernames=()
row_num=1

# Skip header, process each row
tail -n +2 "$MAPPING" | while IFS=',' read -r device_id username; do
  row_num=$((row_num + 1))
  device_id=$(echo "$device_id" | tr -d '\r' | xargs)
  username=$(echo "$username" | tr -d '\r')

  # Check for leading/trailing whitespace in username
  if [[ "$username" != "${username#"${username%%[![:space:]]*}"}" || "$username" != "${username%"${username##*[![:space:]]}"}" ]]; then
    noted_usernames+=("$row_num:$device_id")
  fi

  # Trim whitespace
  username=$(echo "$username" | xargs)

  # Build and send request
  json_data="{\"device_id\": \"$device_id\", \"new_auth_username\": \"$username\"}"
  response=$(curl -sk -o /dev/null -w "%{http_code}" \
    -H "Authorization: $ENCODED_TOKEN" \
    -H "Content-Type: application/json" \
    --data "$json_data" \
    "https://$HOST:$PORT/api/device_auth_username")

  if [[ "$response" -ge 200 && "$response" -lt 300 ]]; then
    if [[ "$response" -ne 204 ]]; then
      unexpected_success+=("$device_id")
    else
      echo "Successfully updated device: $device_id"
    fi
  else
    failures+=("$device_id")
  fi
done

# Reporting
if [[ ${#unexpected_success[@]} -gt 0 ]]; then
  echo "\n[WARN] Unexpected success codes for:"
  for d in "${unexpected_success[@]}"; do echo "  $d"; done
fi

if [[ ${#failures[@]} -gt 0 ]]; then
  echo "\n[ERROR] Failed to update:"
  for d in "${failures[@]}"; do echo "  $d"; done
fi

if [[ ${#noted_usernames[@]} -gt 0 ]]; then
  echo "\n[NOTICE] Usernames with surrounding whitespace:"
  for entry in "${noted_usernames[@]}"; do
    echo "  Row ${entry%%:*} -> ${entry#*:}"
  done
fi
